﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CutawaySurface : MonoBehaviour
{

    public GameObject toilet;
    public GameObject bed;
    public GameObject plant;
    public GameObject table;
    public GameObject chair;
    public GameObject tv;
    public GameObject cube;
    public GameObject fridge; 

    private GameObject[] objectsOfInterest;

    private Shader depthShader;
    private Shader standardShader;

    public Material jumpFloodMat;
    public Material prepareForJumpFloodMat;

    private int k;
    private int N;
    private int n;
    private int end = 0;  //false
    private RenderTexture firstDepthTexture;
    private RenderTexture resultTexture1;
    private RenderTexture resultTexture1Depth;
    private RenderTexture resultTexture2;
    private RenderTexture resultTexture2Depth;
  
    private bool resTex1 = false;

    // Start is called before the first frame update
    /// <summary>
    /// Initialize textures and shaders at the start.
    void Start()
    {
        this.GetComponent<Camera>().depthTextureMode = DepthTextureMode.Depth;

        objectsOfInterest = new GameObject[8]
        {
            toilet,
            bed,
            plant,
            table,
            chair,
            tv,
            cube,
            fridge
        };

        depthShader = Shader.Find("Custom/DepthShader");
        standardShader = Shader.Find("Standard");

        //set render textures
        int heightTex = (int)this.GetComponent<Camera>().pixelHeight;
        int widthTex = (int)this.GetComponent<Camera>().pixelWidth;

        firstDepthTexture = new RenderTexture(widthTex, heightTex, 0, RenderTextureFormat.ARGBFloat);
        firstDepthTexture.dimension = UnityEngine.Rendering.TextureDimension.Tex2D;

        resultTexture1 = new RenderTexture(widthTex, heightTex, 0, RenderTextureFormat.ARGBFloat);
        resultTexture1Depth = new RenderTexture(widthTex, heightTex, 0, RenderTextureFormat.RFloat);
        resultTexture2 = new RenderTexture(widthTex, heightTex, 0, RenderTextureFormat.ARGBFloat);
        resultTexture2Depth = new RenderTexture(widthTex, heightTex, 0, RenderTextureFormat.RFloat);
        resultTexture1.dimension = UnityEngine.Rendering.TextureDimension.Tex2D;
        resultTexture1Depth.dimension = UnityEngine.Rendering.TextureDimension.Tex2D;
        resultTexture2.dimension = UnityEngine.Rendering.TextureDimension.Tex2D;
        resultTexture2Depth.dimension = UnityEngine.Rendering.TextureDimension.Tex2D;

        // calculate values n, N  
        // N = number of loops
        n = Mathf.Max(widthTex, heightTex);
        N = (int)Mathf.Ceil(Mathf.Log(n, 2));
    }

    // Update is called once per frame
    /// <summary>
    /// Setting the shaders of the materials of all chosen objects of interest to the depthShader before rendering.
    void Update()
    {
        // First render all objects of interest into firstDepthTexture with depthShader
        this.GetComponent<Camera>().targetTexture = firstDepthTexture;
        foreach (GameObject go in objectsOfInterest)
        {
            if (go.layer == 8) // = ObjectsOfInterest layer
            {
                foreach(Renderer rend in go.GetComponentsInChildren<Renderer>())
                {
                    for (int i = 0; i < rend.materials.Length; i++)
                    {
                        rend.materials[i].shader = depthShader;
                    }
                }
            }
        }

        // Camera render is done after Update() 
    }

    /// <summary>
    /// Setting the shaders of the materials of all chosen objects of interest back to standardShader after rendering.
    void OnPostRender()
    {
        // AFTER Camera render reset shaders of objects of interest back to standardShader
        foreach (GameObject go in objectsOfInterest)
        {
            if (go.layer == 8) // = ObjectsOfInterest layer
            {
                foreach (Renderer rend in go.GetComponentsInChildren<Renderer>())
                {
                    for (int i = 0; i < rend.materials.Length; i++)
                    {
                        rend.materials[i].shader = standardShader;
                    }
                }
            }
        }
    }

    /*
     * Note that a call to Blit with source and dest set to the same RenderTexture may result in undefined behaviour. 
     * A better approach is to either use Custom Render Textures with double buffering, 
     * or use two RenderTextures and alternate between them to implement double buffering manually. https://docs.unity3d.com/ScriptReference/Graphics.Blit.html
     */
    /// <summary>
    /// Contains the jump flood algorithm. Renders the image multiple times in a for-loop and then stores the result texture. 
    /// </summary>
    /// @param source Input render texture.
    /// @param destination Output texture stored in target texture. 
    void OnRenderImage(RenderTexture source, RenderTexture destination)
    {
        // First call prepareForJumpFlood 
        // from firstDepthTexture to resultTexture2 and resultTexture2Depth
        prepareForJumpFloodMat.SetInt("_WriteDepth", 1);
        Graphics.Blit(firstDepthTexture, resultTexture2Depth, prepareForJumpFloodMat);

        prepareForJumpFloodMat.SetInt("_WriteDepth", 0);
        Graphics.Blit(firstDepthTexture, resultTexture2, prepareForJumpFloodMat);

        //do the jump flood algorithm
        for (int i = 0; i < N; i++)
        {         
            //calculate and set k
            k = (int)(n / (Mathf.Pow(2, (i + 1))));
            jumpFloodMat.SetInt("_k", k);

            if (i == 0)
            {
                end = 0;
                jumpFloodMat.SetInt("_End", end);
                jumpFloodMat.SetTexture("_InputTex", resultTexture2);
                jumpFloodMat.SetTexture("_InputDepthTex", resultTexture2Depth);

                jumpFloodMat.SetInt("_WriteDepth", 1);
                Graphics.Blit(resultTexture2, resultTexture1Depth, jumpFloodMat);
                jumpFloodMat.SetInt("_WriteDepth", 0);
                Graphics.Blit(resultTexture2, resultTexture1, jumpFloodMat);

                resTex1 = true;
            }
            else if (i < (N - 1))
            {
                //change between resultTextures, as mentioned above
                if (i % 2 == 1) //odd   i=1,3,5,7,...
                {
                    jumpFloodMat.SetTexture("_InputTex", resultTexture1);
                    jumpFloodMat.SetTexture("_InputDepthTex", resultTexture1Depth);
                    resTex1 = false;

                    jumpFloodMat.SetInt("_WriteDepth", 1);
                    Graphics.Blit(resultTexture1, resultTexture2Depth, jumpFloodMat);
                    jumpFloodMat.SetInt("_WriteDepth", 0);
                    Graphics.Blit(resultTexture1, resultTexture2, jumpFloodMat);

                }
                else   //even i=2,4,6,...
                {
                    jumpFloodMat.SetTexture("_InputTex", resultTexture2);
                    jumpFloodMat.SetTexture("_InputDepthTex", resultTexture2Depth);
                    resTex1 = true;

                    jumpFloodMat.SetInt("_WriteDepth", 1);
                    Graphics.Blit(resultTexture2, resultTexture1Depth, jumpFloodMat);
                    jumpFloodMat.SetInt("_WriteDepth", 0);
                    Graphics.Blit(resultTexture2, resultTexture1, jumpFloodMat);
                }
            }
            else if (i == (N - 1))   //last one
            {
                end = 1;
                jumpFloodMat.SetInt("_End", end);

                if (resTex1 == true)
                {
                    jumpFloodMat.SetTexture("_InputTex", resultTexture1);
                    jumpFloodMat.SetTexture("_InputDepthTex", resultTexture1Depth);

                    jumpFloodMat.SetInt("_WriteDepth", 1);
                    Graphics.Blit(resultTexture1, resultTexture2Depth, jumpFloodMat);
                    this.GetComponent<Camera>().targetTexture = resultTexture2Depth;
                }
                else
                {
                    jumpFloodMat.SetTexture("_InputTex", resultTexture2);
                    jumpFloodMat.SetTexture("_InputDepthTex", resultTexture2Depth);

                    jumpFloodMat.SetInt("_WriteDepth", 1);
                    Graphics.Blit(resultTexture2, resultTexture1Depth, jumpFloodMat);
                    this.GetComponent<Camera>().targetTexture = resultTexture1Depth;
                }
            }
        }
    }
}
