﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using TMPro;
using System;


public class SettingsScript : MonoBehaviour
{
    // Objects of interest
    public GameObject toilet;
    public GameObject bed;
    public GameObject plant;
    public GameObject table;
    public GameObject chair;
    public GameObject tv; 
    public GameObject cube;
    public GameObject fridge; 
    private GameObject[] objectsOfInterest;

    // Shader objects
    private Shader cutShader;
    private Shader standardShader;  // From unity

    // Toggles objects of interest
    public GameObject toggleCubeGO;
    public GameObject toggleToiletGO;
    public GameObject toggleBedGO;
    public GameObject togglePlantGO;
    public GameObject toggleChairGO;
    public GameObject toggleTableGO;
    public GameObject toggleTvGO;
    public GameObject toggleFridgeGO;

    // Toggles improvements
    public GameObject togglePerspCompGO;
    public GameObject toggleEdgeComprGO;
    public GameObject toggleDirConstrGO;
    private bool useEdgeCompr = true;
    private bool useDirConstr = false;

    // For theta angle settings
    public Material jumpFloodMat;
    public GameObject textValue;
    public Slider slider;
    private float currentTheta;

    // Start is called before the first frame update
    /// <summary>
    /// Set shaders and gameobjects at the start. 
    void Start()
    {
        objectsOfInterest = new GameObject[8]
        {
            toilet,
            bed,
            plant,
            table,
            chair,
            tv,
            cube,
            fridge
        };

        cutShader = Shader.Find("Custom/CutShader");
        standardShader = Shader.Find("Standard");

        //set slider to value of material
        setAngleValue(55.0f);
    }

    /// <summary>
    /// Update is called once per frame.
    void Update()
    {
        // set value in jump flood material
        jumpFloodMat.SetFloat("_Theta", currentTheta);

        // set value in depth shader
        foreach (GameObject go in objectsOfInterest)
        {
            if (go.layer == 8) // = ObjectsOfInterest layer
            {
                foreach (Renderer rend in go.GetComponentsInChildren<Renderer>())
                {
                    for (int i = 0; i < rend.materials.Length; i++)
                    {
                        rend.materials[i].SetFloat("_Theta", currentTheta);
                        rend.materials[i].SetInt("_UseEdgeCompression", useEdgeCompr ? 1 : 0);
                        rend.materials[i].SetInt("_UseDirectionalConstraints", useDirConstr ? 1 : 0);
                    }
                }
            }
        }
    }

    /// <summary>
    /// Method for toggle the cube as object of interest.
    public void toggleCube()
    {
        toggleObjectOfInterest(cube, toggleCubeGO);
    }

    /// <summary>
    /// Method for toggle the bed as object of interest.
    public void toggleBed()
    {
        toggleObjectOfInterest(bed, toggleBedGO);
    }

    /// <summary>
    /// Method for toggle the plant as object of interest.
    public void togglePlant()
    {
        toggleObjectOfInterest(plant, togglePlantGO);
    }

    /// <summary>
    /// Method for toggle the toilet as object of interest.
    public void toggleToilet()
    {
        toggleObjectOfInterest(toilet, toggleToiletGO);
    }

    /// <summary>
    /// Method for toggle the table as object of interest.
    public void toggleTable()
    {
        toggleObjectOfInterest(table, toggleTableGO);
    }

    /// <summary>
    /// Method for toggle the chair as object of interest.
    public void toggleChair()
    {
        toggleObjectOfInterest(chair, toggleChairGO); 
    }

    /// <summary>
    /// Method for toggle the tv as object of interest.
    public void toggleTV()
    {
        toggleObjectOfInterest(tv, toggleTvGO);
    }

    /// <summary>
    /// Method for toggle the fridge as object of interest.
    public void toggleFridge()
    {
        toggleObjectOfInterest(fridge, toggleFridgeGO);
    }


    /// <summary>
    /// Change settings of toggled objects.
    /// 
    /// Changes the layer and the shader of all materials of all toggled objects and childs. 
    /// @param go The gameobject which should be toggled.
    /// @param toggleGO The GUI-toggle element. 
    private void toggleObjectOfInterest(GameObject go, GameObject toggleGO)
    {
        Renderer[] rends = go.GetComponentsInChildren<Renderer>();
        Shader newShader;
        int layer;

        if (toggleGO.GetComponent<Toggle>().isOn)
        {
            newShader = standardShader;
            go.layer = 8;   //layer = object of interest
            layer = 8;
        }
        else
        {
            go.layer = 0;  //layer = default
            layer = 0;
            newShader = cutShader;
        }

        // Set layer of all children of gameObject
        Transform[] childs = go.GetComponentsInChildren<Transform>();
        foreach (Transform child in childs)
        {
            child.gameObject.layer = layer;
        }

        // Set shader in each child-GO
        foreach (Renderer rend in rends)
        {
            // Set shader for all assigned materials
            for (int i = 0; i < rend.materials.Length; i++)
            {
                rend.materials[i].shader = newShader;
            }
        }
    }


    /// <summary>
    /// Method for setting the edge compression
    public void toggleEC()
    {
        useEdgeCompr = toggleEdgeComprGO.GetComponent<Toggle>().isOn;
    }

    /// <summary>
    ///Method for setting the directional contrainst
    public void toggleDC()
    {
        useDirConstr = toggleDirConstrGO.GetComponent<Toggle>().isOn;
    }

    /// <summary>
    /// Set value of angle in the GUI.
    public void setAngleValue(float val)
    {
        // convert from degrees to radians
        currentTheta = val * Mathf.PI / 180.0f;

        //set text of actual angle
        textValue.GetComponent<TextMeshProUGUI>().SetText(System.Math.Round(val, 1).ToString());
    }




//this is the mainPage for the documentation:
    /**
\mainpage Adaptive Cutaways Documentation
        
\authors Michaela Niedermayer, Alexander Heinz


This is the source documentation of our application, which is an implementation of the paper "Adaptive Cutaways for Comprehensible
Rendering of Polygonal Scenes" by Burns et al.[1]. It is used to expose objects of interest, which are occluded by not so important objects through the creation of a view-dependent cutaway surface.The application was implemented with Unity (version 2019.3). 
       *
       * 
    *<img src="screenshotNear.PNG" align="left" caption="Cutaway to see objects of interest">
     * <div style="clear: both"></div>
     
<br>
[1] Burns, Michael, and Adam Finkelstein. "Adaptive cutaways for comprehensible rendering of polygonal scenes." ACM transactions on graphics (TOG) 27.5 (2008): 1-7.


*/
}
