﻿Shader "Custom/DepthShader"
{
    Properties
    {
        _MainTex ("Texture", 2D) = "white" {}
        _Theta("Theta Angle", float) = 0
        _UseEdgeCompression("Use Edge Compression", int) = 0
        _UseDirectionalConstraints("Use Directional Constraints", int) = 0
    }
    SubShader
    {
        Tags { "RenderType"="Opaque" }
		Cull Front

        Pass
        {
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag

            #include "UnityCG.cginc"

            struct appdata
            {
                float4 vertex : POSITION;
            };

            struct v2f
            {
                float4 vertex : SV_POSITION;
				float3 viewPosition : TEXCOORD1;
                float4 screenPos : TEXCOORD2;
            };

            float _Theta;
            int _UseEdgeCompression;
            int _UseDirectionalConstraints;

            v2f vert (appdata v)
            {
                v2f o;
                o.vertex = UnityObjectToClipPos(v.vertex);
				o.viewPosition = UnityObjectToViewPos(v.vertex);
                o.screenPos = ComputeScreenPos(o.vertex);
                return o;
            }

            fixed4 frag (v2f i) : SV_Target
            {
                // Store linear depth in r
                // Store 1 in g to determine, which fragments are written
                // Store theta with edge compression in b
                // Store theta with directional constraints in a
                float2 screenUV = i.screenPos.xy / i.screenPos.w;
                float zNear = 0.01f;
                float zFar = 300.0f;
                float linearDepth =  (-i.viewPosition.z - zNear) / (zFar - zNear);

                float thetaEdgeCompr = _Theta;
                float thetaDirConstr = _Theta;

                if (_UseEdgeCompression == 1) {
                    float r = 0.1f;
                    float2 ndc = screenUV * 2.0f - 1.0f; // [0, 1] -> [-1, 1]
                    float x = clamp((1.0f - abs(ndc.x)) / r, 0, 1);
                    float y = clamp((1.0f - abs(ndc.y)) / r, 0, 1);
                    thetaEdgeCompr = _Theta * x * y;
                    thetaDirConstr = thetaEdgeCompr;
                }

                if (_UseDirectionalConstraints == 1) {
                    float PI = 3.14159265358979323846f;
                    float phi = (PI / 2.0f) - acos(dot(normalize(-i.viewPosition), float3(0, 1, 0)));
					thetaDirConstr = clamp(thetaEdgeCompr, 0, abs(phi)) + thetaEdgeCompr * smoothstep(0, PI / 18.0f, -phi);
                }

                return fixed4(linearDepth, 1, thetaEdgeCompr / _Theta, thetaDirConstr / _Theta);
            }
            ENDCG
        }
    }
}
