﻿Shader "Custom/JumpFloodShader"
{
    Properties
    {
        _MainTex ("Texture", 2D) = "white" {}
		_Theta("Theta Angle", float) = 0
		_InputTex("Result Texture from before", 2D) = "black" {}
		_InputDepthTex("Result Depth Texture from before", 2D) = "black" {}
		_k("Jump Length k", int) = 0
		_End("Last Loop", int) = 0		
		_WriteDepth("Write Depth", int) = 0
    }
    SubShader
    {
        Pass
        {
            CGPROGRAM
			
            #pragma vertex vert
            #pragma fragment frag

            #include "UnityCG.cginc"
					
		    sampler2D _MainTex;
			uniform float4 _MainTex_TexelSize;
			sampler2D _InputTex;
			sampler2D _InputDepthTex;
			float _Theta;
			int _k;
			int _End;
			int usePerspectiveCompensation =1;
			int _WriteDepth;

            struct appdata
            {
                float4 vertex : POSITION;
                float2 uv : TEXCOORD0;
				float4 screenPos : TEXCOORD1; 
            };

            struct v2f
            {
                float4 vertex : SV_POSITION;
				float4 screenPos : TEXCOORD1;
            };

            v2f vert (appdata v)
            {
				v2f o;							
				o.vertex = UnityObjectToClipPos(v.vertex);
				o.screenPos = ComputeScreenPos(o.vertex); 
                return o;
            }
          
			fixed4 frag(v2f i) : SV_Target
			{
				float zNear = 0.01f;
				float zFar = 300.0f;
				float4 newTexSample;
				float zq;
				float C; 
							
				// Start here with calulcations for Cutaway Surface

				// p ... current screen position					
				float2 p = i.screenPos.xy / i.screenPos.w;
				float zp = tex2D(_InputDepthTex, p).r;
				
				// cMax = max cutaway function
				float cMax = -1.0f;
				float newDepth = zp;

				// texture contains screenPos of max depth
				newTexSample = tex2D(_InputTex, p);
				
				// go from p in all directions with the changing length of k				
				float2 candidatePoints[9];
				candidatePoints[0] = float2(-_k, _k); //top right
				candidatePoints[1] = float2(0, _k);   //middle top
				candidatePoints[2] = float2(_k, _k);  //left top
				candidatePoints[3] = float2(-_k, 0);  //middle right
				candidatePoints[4] = float2(0, 0);   //middle middle
				candidatePoints[5] = float2(_k, 0);   //right middle
				candidatePoints[6] = float2(-_k, -_k);  //left bottom
				candidatePoints[7] = float2(0, -_k);   //middle bottom
				candidatePoints[8] = float2(_k, -_k);  //right bottom - ist aber left bottom

				// Scale for slope m with Perspective Compensation	
				float pCompScale =  (zNear + zFar) / (zNear - zFar);

				[loop]
				for (int j = 0; j < 9; j++) {
					float2 coords = ((p * _MainTex_TexelSize.zw) + candidatePoints[j]) / _MainTex_TexelSize.zw;

					//all coords have to lie between 0 and 1
					if (coords.x > 0 && coords.y > 0 && coords.x < 1 && coords.y < 1) {
						zq = tex2D(_InputDepthTex, coords).r;
						float4 texSample = tex2D(_InputTex, coords);
						float2 q = texSample.xy;  
																			
						float thetaEdge = texSample.z * _Theta; // [0, 1] -> [0, _Theta]
						float thetaDir = texSample.w * _Theta; // [0, 1] -> [0, _Theta]

						float mEdgeComp = 1.0f / tan(thetaEdge);
						float mDirConstr = 1.0f / tan(thetaDir);

						
						if (usePerspectiveCompensation == 1) {
							mEdgeComp *= -(pCompScale + zq);
							mDirConstr *= -(pCompScale + zq);
						}
						
						float alpha = clamp(dot(normalize(q - p), float2(0.0f, 1.0f)), 0.0f, 1.0f);
						float m = ((1.0f - alpha) * mEdgeComp + alpha * mDirConstr);
						C = zq - m * distance(q, p);

						if (C > cMax) {							
							cMax = C;
							newDepth = zq;
							newTexSample = texSample;							
						}
					}
				}

				if (_End == 1) {
					float2 q = newTexSample.xy;
					float z = newDepth;

					float thetaEdge = newTexSample.z * _Theta; // [0, 1] -> [0, _Theta]
					float thetaDir = newTexSample.w * _Theta; // [0, 1] -> [0, _Theta]
				
					float mEdgeComp = 1.0f / tan(thetaEdge);
					float mDirConstr = 1.0f / tan(thetaDir);
					if (usePerspectiveCompensation == 1) {
						mEdgeComp *= -(pCompScale + z);
						mDirConstr *= -(pCompScale + z);
					}

					float alpha = clamp(dot(normalize(q - p), float2(0.0f, 1.0f)), 0.0f, 1.0f);
					float m = ((1.0f - alpha) * mEdgeComp + alpha * mDirConstr);
					newDepth = z - m * distance(q, p);
				}	

				if (_WriteDepth == 1) {
					return fixed4(newDepth, 0, 0, 0);
				}
				else {
					return newTexSample;
				} 
            }
            ENDCG
        }
    }
}
