﻿Shader "Custom/CutShader"
{
	Properties
	{
		_MainTex("Albedo (RGB)", 2D) = "white" {}
		_Color("Color", Color) = (1,1,1,1)
		_JumpFloodDepthTex("Jump Flood Depth Texture", 2D) = "black" {}
	}
		SubShader
	{
		Pass
		{
			Tags {"LightMode" = "ForwardBase"}

			Cull off
			
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag
			#include "UnityCG.cginc"
			#include "UnityLightingCommon.cginc"

			struct v2f
			{
				float2 uv : TEXCOORD0;
				float3 normal : TEXCOORD3;
				float4 vertex : SV_POSITION;
				float3 viewPosition : TEXCOORD1;
				float4 screenPos : TEXCOORD2; //screen position of p
			};

			fixed4 _Color;
			sampler2D _JumpFloodDepthTex;
			sampler2D _MainTex;
			uniform float4 _MainTex_TexelSize;
			
			v2f vert(appdata_base v)
			{
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);	
				o.viewPosition = UnityObjectToViewPos(v.vertex);
				o.screenPos = ComputeScreenPos(o.vertex);				
				o.uv = v.texcoord;  	
				o.normal = normalize(UnityObjectToWorldNormal(v.normal)); // COMPUTE_VIEW_NORMAL;
				//half nl = max(0, dot(worldNormal, _WorldSpaceLightPos0.xyz));
				//o.diff = nl * _LightColor0;
				// the only difference from previous shader:
				// in addition to the diffuse lighting from the main light,
				// add illumination from ambient or light probes
				// ShadeSH9 function from UnityCG.cginc evaluates it,
				// using world space normal
				//o.ambient.rgb = ShadeSH9(half4(worldNormal, 1));
				return o;
			}

			struct fout
			{
				fixed4 color : SV_Target;
				float depth : SV_DEPTH;
			};

			fout frag(v2f i, fixed facing : VFACE) : SV_Target
			{
				fout ret;
				float zNear = 0.01f;
				float zFar = 300.0f;
				fixed2 screenUV = i.screenPos.xy / i.screenPos.w;
				float jumpFloodDepth = tex2D(_JumpFloodDepthTex, screenUV).r;
				float surfaceDepth = (-i.viewPosition.z - zNear) / (zFar - zNear);
				if (jumpFloodDepth > surfaceDepth) {
					//clip(-1);
					discard;
				}

				fixed4 col = tex2D(_MainTex, i.uv) * _Color;
				float3 viewPosition = i.viewPosition;
				float3 normal = normalize(i.normal);
				float3 viewNormal = normalize(mul(UNITY_MATRIX_V, float4(normal, 0.0f)));

				float4 clipPosition = mul(UNITY_MATRIX_P, float4(viewPosition, 1.0f));
				ret.depth = clipPosition.z / clipPosition.w;

				if (facing <= 0) {
					// Render cut surface
					// Project visible backfaces into cutting volume for illumination
					float volumeViewDepth = -zNear - jumpFloodDepth * (zFar - zNear);
					float3 volumeViewPosition = float3(screenUV *
						-volumeViewDepth / float2(UNITY_MATRIX_P[0][0], UNITY_MATRIX_P[1][1]), volumeViewDepth);
					viewNormal = normalize(cross(ddx(volumeViewPosition), ddy(volumeViewPosition)));
					col = tex2D(_MainTex, i.uv) * _Color;

					float4 volumeClipPosition = mul(UNITY_MATRIX_P, float4(volumeViewPosition, 1.0f));
					ret.depth = volumeClipPosition.z / volumeClipPosition.w;
				}

				float3 toLight = normalize(_WorldSpaceLightPos0.xyz);
				float3 viewToLight = normalize(mul(UNITY_MATRIX_V, float4(toLight, 0.0f)));

				float ambient = 0.3f;
				float diffuse = max(dot(viewToLight, viewNormal), 0.0f);

				ret.color = ambient * col + (_LightColor0 *  diffuse * col); 
				return ret;
			}
			ENDCG
		}
	}
}